// require('coffee-script/register');
var util         = require("util");
var EventEmitter = require("events").EventEmitter;

var Stage 		= require("../../app/models/stage.js");
var ChoicesFlow = require("../../app/models/choicesFlow.js");
var JobRunner   = require("../../app/runner/jobRunner.js");
var Sync 		= require("sync");
var async 		= require("async");
var authUtil 	= require('../../app/utils/authUtil.js');
var winston 	= require('winston');
var merge 		= require("merge");

var Flow = require("../../app/helpers/executions/flow.coffee");
var SSH = require("../../app/helpers/executions/ssh.coffee");
var SCP = require("../../app/helpers/executions/scp.coffee");
var Local = require("../../app/helpers/executions/local.coffee");
var Context = require("../../app/helpers/executions/context.coffee");
var CommandsModel = require("../../app/helpers/executions/commands.coffee");
var Command = require("../../app/helpers/executions/command.coffee");

var Environment = require("../../app/models/environment2.js");
var Visualizations = require("../../app/helpers/visualizations/visualizations.js");
var Commands = require("../../app/helpers/commands/commands.js");
var Questions = require("../../app/helpers/questions/questions.js");
var QuestionsLoader = require("../../app/helpers/questions/questionsLoader.js");

//Responsible for handle job execution
//====================
function SherlockContext(options){
	this.console = console,
	this.require = require,
	this.process = process,
	this.setInterval = setInterval,
	this.context = {
		environment: Environment,
		visualizations:Visualizations,
		commands: Commands,
		investigations: [],
		questions: Questions,
		questionsLoader: QuestionsLoader,
		conclusions: [],
		executions: {
			Flow: Flow,
			SSH: SSH,
			SCP: SCP,
			Local: Local,
			Context: Context,
			Commands: CommandsModel,
			Command: Command
		}
	}

	var instance = this;
	this.skip = skipFN;
	this.investigation=function(name, callback){
		investigationFN(name, callback, instance);
	};

	this.investigate=function(name, callback){
		investigateFN(name, callback, instance);
	};

	this.ask=function(questionsToAsk, callback, properties){
		askFN(questionsToAsk, callback, properties, instance);
	};

	this.check=function(context, callback){
		checkFN(context, callback, instance);
	};

	this.conclusion=function(conclusionObj, callback){
		conclusionFN(conclusionObj, callback, instance);
	};

	this.conclude=function(callback){
		concludeFN(callback, instance);
	};
	EventEmitter.call(this);
}

//Skip investigate function
var skipFN={
	investigate: function(name, callback){
			console.log(" - Skipped investigate : "+ name);
	}
};

//Main function responsible for load job
var investigationFN= function(name, callback, instance){
	try{
		console.log("####### Investigation running: "+name);
		callback(instance.context);
	}catch(err){
		winston.error('Problem in investigation: ', err);
		throw err;
	}
};

//Responsible for load all investigate declarations and add in a array
var investigateFN=function(name, callback, instance){
	try{
		winston.debug("Investigate");
		instance.context.investigations.push({name:name, callback:callback});
	}catch(err){
		winston.error('Problem in investigate: ', err);
		throw err;
	}
};

//Responsible for prompt questions to user
var askFN=function(questionsToAsk, callback, properties, instance){
	try{
		winston.debug("Receiving default properties: ", properties);
		if(!properties || properties == undefined){
			properties={};
		}
		if(!questionsToAsk || questionsToAsk.length ==0){
			winston.debug("There are no questions. Sherlock will call callback with default properties: ", properties);
			callback(properties);
			return;
		}

		var constructorProperties = merge(properties, instance.answers);
		var resultCallback = callback;
		var stage = new Stage("#", constructorProperties);
		var flow = new ChoicesFlow();
		questionsToAsk.forEach(function(question){
			stage.addQuestion(question);
		});

		//Configuring subStage if have propertie specialCommands in answers
		stage.after(function(answers, callback){
			if(answers['specialCommands'] && answers['specialCommands'].length > 0){
				answers.component = answers['specialCommands'][0].component;
				var subStage = new Stage("SubStage", answers);
				specialCommands = answers['specialCommands'];
				specialCommands.forEach(function(command){
					if(!command.questions){
						command.questions=[];
					}
					command.questions.forEach(function(question){
						var questionFN = instance.context.questions[question];
						subStage.addQuestion(questionFN());
					});
				});

				if(subStage.questions.length > 0){
					this.controller.registerStage(subStage, answers);
				}
			}
			callback();
		});

		flow.registerStage(stage, constructorProperties);

		flow.start(function(err, answers){
			if(err){
				winston.error("Error: "+ err);
			}

			if(answers.specialCommands){
				answers.commands=[];
				answers.specialCommands.forEach(function(command){
					try{
						var instanceCMD = require(process.cwd()+"/./"+command.file);
						answers = merge(answers, command.params);
						var rawCommand = instanceCMD(answers, instance.context.environment);
						var commandObject = {name: command.name, command:rawCommand , params: command.params};
						answers.commands.push(commandObject);
					}catch(err){
						console.log("Problem to build command: "+ err);
					}
				});
			}

			instance.answers=answers;
			resultCallback(answers);
		});
	}catch(err){
		winston.error('Problem in ask: ', err);
		throw err;
	}
};

//Access by SSH configurated servers
var checkFN= function(context, callback, instance){
	try{
		if(context instanceof Flow){
			var flow = context;
			flow.on('end_flow', function(err, results){
        callback(results);
      })
      flow.execute();
			return;
		}

		context.username=authUtil.getUsername();
		context.password=authUtil.getPassword();
		var runner = new JobRunner(context);
		runner.run();

		var ProgressBar = require('progress');
		var bar = new ProgressBar(context.report.name + '\t' + context.servers + ' - [:bar] :percent ', { total: 100 });
		bar.tick(0);

		if(context.stream){
			runner.on('data', function(results){
				callback(results);
			});
		}

		runner.on('execution', function(percent){
			bar.tick(percent.percent);
		});

		runner.on('start', function(data){
			winston.info("Starting access to servers");
		});

		runner.on('consolidate', function(results){
			delete context['username'];
			delete context['password'];
			winston.debug("Receiving data from servers:");
			winston.debug(results);
			callback(results);
		});
	}catch(err){
		winston.error('Problem in check: ', err);
		throw err;
	}
};

//Reponsible for add a conclusion in sherlock context
var conclusionFN= function(conclusion, callback, instance){
	try{
		winston.debug("Conclusion");
		var response = {
			context: instance.answers,
			conclusion: conclusion
		}
		instance.context.conclusions.push(response);
		callback(response);
	}catch(err){
		winston.error('Problem in conclusion: '+err, err);
		throw err;
	}
};

//It's the last function called and responsible for execute all investigate added
var concludeFN=function(callback, instance){
	winston.debug("Starting investigations in conclude");
	try{
		async.eachSeries(instance.context.investigations, function(investigate, callback){
			investigate.callback(instance.context, callback);
		}, function(err){
			if(err){
				winston.error('Problem in conclude: ', err);
			}
			callback(instance.context.conclusions);
			winston.debug("[SherlockContext.js] Emiting 'end' event");
			instance.emit('end', instance.context.conclusions);
		});
	}catch(err){
		winston.error('Problem in conclude: ', err);
		throw err;
	}
};

util.inherits(SherlockContext, EventEmitter);
//var instance = new SherlockContext();
module.exports=SherlockContext;