var winston 	 = require('winston');
var environment  = require("../../models/environment2.js");
var date 		 = new Date();
var timeFormated = date.toTimeString().split(" ")[0].substring(0, 5);
var dateArray	 = date.toDateString().split(" ");
var dateFormated = dateArray[2] + "/" + dateArray[1] + "/" + dateArray[3];
var _ 			 = require("underscore");
var inquirer 	 = require("inquirer");
var moment = require("moment");

var shouldApply = function(definition, answers){
	var should = !answers[definition.name];
	winston.debug('Should ask question? ', !answers[definition.name]);
	if(!should){
		winston.info(definition.message + " = " + answers[definition.name]);
	}
	return should;
}

// This Is A Header
// ----------------

/**
* Default question about the environment where the jobs will run.
* @param {string} properties.message - Message to prompt
* @param {string} properties.name - name of attribute in json
* @param {string} properties.type - type of question, can be: input, list, checkbox
* @param {string} properties.default - default value to this field
* @param {string} properties.choices - if the type is list or checkbox, this attribute specify the options showed to users
*/
module.exports.informTheEnvironment = function(properties){
	if(!properties){
		properties={};
	}

	var questionDefinition={};
	questionDefinition.message = properties.message || 'Inform the Environment';
	questionDefinition.name    = properties.name 	|| 'env';
	questionDefinition.type    = properties.type 	|| 'list';
	questionDefinition.default = properties.default || 'prod';
	questionDefinition.choices = properties.choices || environment.getEnvironmentList();

	return function(answers){
		return {
			message: questionDefinition.message,
			name: questionDefinition.name,
			type:questionDefinition.type,
			choices:questionDefinition.choices,
			default: questionDefinition.default,
			apply: function(answers){
				return shouldApply(questionDefinition, answers);
			}
		};
	}
}

module.exports.informTheComponent = function(properties){
	if(!properties){
		properties={};
	}

	var questionDefinition={};
	questionDefinition.message = properties.message || 'Inform the Component';
	questionDefinition.name    = properties.name 	|| 'component';
	questionDefinition.type    = properties.type 	|| 'list';
	questionDefinition.default = properties.default || 'saas';

	return function(answers){
		return {
			message: questionDefinition.message,
			name: questionDefinition.name,
			type:questionDefinition.type,
			choices:environment.getComponentList(answers.env),
			default: questionDefinition.default,
			apply: function(answers){
				return shouldApply(questionDefinition, answers);
			}
		};
	}
}

module.exports.selectServers = function(properties){
	if(!properties){
		properties={};
	}

	var questionDefinition={};
	questionDefinition.message = properties.message || 'Select Servers';
	questionDefinition.name    = properties.name 	|| 'servers';
	questionDefinition.type    = properties.type 	|| 'checkbox';

	return function(answers){
		winston.debug('Returning question selectServers with answers: ', answers);
		return {
			message: questionDefinition.message,
			name: questionDefinition.name,
			type:questionDefinition.type,
			choices: environment.getComponent(answers.env, answers.component).servers,
			default: environment.getComponent(answers.env, answers.component).servers,
			apply: function(answers){
				return shouldApply(questionDefinition, answers);
			}
		};
	}
}

module.exports.resultSize = function(properties){
	if(!properties){
		properties={};
	}

	var questionDefinition={};
	questionDefinition.message = properties.message || 'Results size';
	questionDefinition.name    = properties.name 	|| 'size';
	questionDefinition.type    = properties.type 	|| 'input';
	questionDefinition.default = properties.default || 5;

	return function(answers){
		return {
			message: questionDefinition.message,
			name:questionDefinition.name,
			type:questionDefinition.type,
			default: questionDefinition.default,
			apply: function(answers){
				return shouldApply(questionDefinition, answers);
			}
		};
	}
}

module.exports.startDate = function(properties){
	if(!properties){
		properties={};
	}

	var questionDefinition={};
	questionDefinition.message = properties.message || 'Start date';
	questionDefinition.name    = properties.name 	|| 'startDate';
	questionDefinition.type    = properties.type 	|| 'input';
	questionDefinition.default = properties.default || dateFormated;

	return function(answers){
		return {
			message: questionDefinition.message,
			name:questionDefinition.name,
			type:questionDefinition.type,
			default: questionDefinition.default,
			apply: function(answers){
				return shouldApply(questionDefinition, answers);
			}
		};
	}
}

module.exports.startTime = function(properties){
	if(!properties){
		properties={};
	}

	var questionDefinition={};
	questionDefinition.message = properties.message || 'Start time';
	questionDefinition.name    = properties.name 	|| 'startTime';
	questionDefinition.type    = properties.type 	|| 'input';
	questionDefinition.default = properties.default ||  "00:00";

	return function(answers){
		return {
			message: questionDefinition.message,
			name:questionDefinition.name,
			type:questionDefinition.type,
			default: questionDefinition.default,
			apply: function(answers){
				return shouldApply(questionDefinition, answers);
			}
		};
	}
}

module.exports.endDate = function(properties){
	if(!properties){
		properties={};
	}

	var questionDefinition={};
	questionDefinition.message = properties.message || 'End date';
	questionDefinition.name    = properties.name 	|| 'endDate';
	questionDefinition.type    = properties.type 	|| 'input';
	questionDefinition.default = properties.default ||  dateFormated;

	return function(answers){
		return {
			message: questionDefinition.message,
			name:questionDefinition.name,
			type:questionDefinition.type,
			default: questionDefinition.default,
			apply: function(answers){
				return shouldApply(questionDefinition, answers);
			}
		};
	}
}

module.exports.endTime = function(properties){
	if(!properties){
		properties={};
	}

	var questionDefinition={};
	questionDefinition.message = properties.message || 'End time';
	questionDefinition.name    = properties.name 	|| 'endTime';
	questionDefinition.type    = properties.type 	|| 'input';
	questionDefinition.default = properties.default ||  timeFormated;

	return function(answers){
		return {
			message: questionDefinition.message,
			name:questionDefinition.name,
			type:questionDefinition.type,
			default: questionDefinition.default,
			apply: function(answers){
				return shouldApply(questionDefinition, answers);
			}
		};
	}
}

module.exports.selectCommands = function(properties){
	if(!properties){
		properties={};
	}

	var questionDefinition={};
	questionDefinition.message = properties.message || 'Select a command';
	questionDefinition.name    = properties.name 	|| 'commands';
	questionDefinition.type    = properties.type 	|| 'checkbox';
	questionDefinition.default = properties.default ||  null;

	return function(answers){
		return {
			message: questionDefinition.message,
			name: questionDefinition.name,
			type: questionDefinition.type,
			choices: environment.getCommands(answers.component),
			filter: function(selecteds){
				var commands = [];
				selecteds.forEach(function(command){
					var cmd = environment.getCommand(answers.component, command);
					for(key in cmd.params){
						winston.debug(key);
						answers[key]=cmd.params[key];
					}
					var result = {name: cmd.name, command: cmd.command(answers, environment)}
					winston.debug("Generated command: ", result);
					commands.push(result);
				});
				return commands;
			},
			apply: function(answers){
				return shouldApply(questionDefinition, answers);
			}
		}
	}
}

module.exports.selectDynamicCommands = function(properties){
	if(!properties){
		properties={};
	}

	var questionDefinition={};
	questionDefinition.message= properties.message 		|| 'Select a command';
	questionDefinition.name   = properties.name 		|| 'specialCommands';
	questionDefinition.type   = properties.type 		|| 'checkbox';
	questionDefinition.default= properties.default 		||  null;
	questionDefinition.prefix = properties.prefixComponent ||  true;
	questionDefinition.error  = properties.errorMessage ||  "The selected commands must be from only one component ";

	return function(answers){
		var commands = environment.getAllCommands();
		var dynamicCommands=[];
		var keys = _.keys(commands);
		keys.forEach(function(component){
			var commandsByComponents = commands[component];
			if(commandsByComponents.length > 0){
				dynamicCommands.push(new inquirer.Separator("=== "+component+" ==="));
			}
			commandsByComponents.forEach(function(command){
				command.component=component;
				if(questionDefinition.prefix){
					command.name = "["+component+"] - "+command.name;
				}
				dynamicCommands.push({name: command.name, value: command});
			});
		});

		return {
			message: questionDefinition.message,
			name: questionDefinition.name,
			type: questionDefinition.type,
			choices: dynamicCommands,
			filter: function(selecteds){
				return selecteds;
			},
			apply: function(answers){
				return shouldApply(questionDefinition, answers);
			},
			validate: function(answers){
				components = answers.map(function(command){
					return command.component
				});

				var isValid = _.uniq(components).length;
				return (isValid == true)? true : questionDefinition.error;
			}
		}
	}
}