var d3 = require('d3');
var path = require('path');
var winston = require('winston');
var readdir = require('readdir');

var genericVisualizations = [];
var localVisualizations =[];

 try{
	module.exports.SimpleTableVisualization = require('./simpleTableVisualization');
	module.exports.TableWithLabeledRowVisualization = require('./tableWithLabeledRowVisualization');
	module.exports.SimpleHeaderVisualization = require('./simpleHeaderVisualization');
	module.exports.MultiCommandTableVisualization = require('./multiCommandTableVisualization');

	module.exports.get=function(visualizationName){
		var path = require("path");
		var visualizationDir = path.join(process.cwd(), "visualizations", visualizationName);
		return require(visualizationDir);
	}

	//console.log('load local genericVisualizations');
	genericVisualizations = readdir.readSync(__dirname, ['**.js']);
	genericVisualizations.forEach(function(visualization){
	 	var normVisualization = path.basename(visualization,".js");
	 	if(normVisualization != "visualizations"){
	 		//console.log('Loading generic visualization: '+normVisualization);
	 		module.exports[normVisualization]=require("./"+visualization);
	 	} 
	});

	var visualizationDir = path.join(process.cwd(), "visualizations");
	localVisualizations = readdir.readSync(visualizationDir, ['**.js']);
	localVisualizations.forEach(function(visualization){
		var normVisualization = path.basename(visualization,".js");
		//console.log('Loading local visualization: '+normVisualization);
		module.exports[normVisualization]=require(visualizationDir+"/./"+visualization);
	});
}catch(err){
	winston.error('Problem to load visualization: ', err);
}

module.exports.genericVisualizations = genericVisualizations;
module.exports.localVisualizations = localVisualizations;