var ChoicesFlow = require("../models/choicesFlow.js");
var Stage 		= require("../models/stage.js");
var Environment = require("../models/environment.js");
var JobRunner   = require("../runner/jobRunner.js");
var flow 		= new ChoicesFlow();
var d3 = require('d3');
var cli = require('cli-color');
var _ = require('underscore');
var Table = require('cli-table');
var numeral = require('numeral');

var date = new Date();
var dateArray = date.toDateString().split(" ");
var dateFormated = dateArray[2] + "/" + dateArray[1] + "/" + dateArray[3];
var timeFormated = date.toTimeString().split(" ")[0].substring(0, 5);
var stage = new Stage("Simple questions");

var env = "";
var component = "";

stage.addQuestion({
	type: "list",
	name: "env",
	message: "Inform the Environment",
	choices: [
		"prod"
	],
	default: 'prod',
	filter: function(answer) {
		env = answer;
		return env;
	}
});

stage.addQuestion({
	type: "list",
	name: "component",
	message: "Inform the components",
	choices: [
		"cache_varnish"
	],
	default: 'cache_varnish',
	filter: function(answer) {
		component = answer;
		return component;
	}
});

stage.addQuestion({
	type: "checkbox",
	name: "servers",
	message: "Select servers",
	choices : function() {
		return Environment[env][component].servers;
	},
	default: function() {
		return Environment[env][component].servers;
	}
});

stage.addQuestion({
	type: "input",
	name: "startDate",
	message: "Start date",
	default: dateFormated
});

stage.addQuestion({
	type: "input",
	name: "startTime",
	message: "Start time",
	default: "00:00"
});

stage.addQuestion({
	type: "input",
	name: "endDate",
	message: "End date",
	default: dateFormated
});

stage.addQuestion({
	type: "input",
	name: "endTime",
	message: "End time",
	default: timeFormated
});

stage.after(function(answers, callback){
	callback();
});
flow.registerStage(stage);

flow.start(function(err, answers){
	//console.log(answers);
	var authUtil = require('../utils/authUtil.js');
	var options={
		servers:  answers.servers,
		username: authUtil.getUsername(),
		password: authUtil.getPassword(),
		report:{
			name:'QPS',
			commands: [
				'cat /var/log/varnish/varnishncsa.log* | grep -v "healthcheck_varnish.html" | awk \'(($4>\"['+answers.startDate+':'+answers.startTime+':00\") && ($4<\"['+answers.endDate+':'+answers.endTime+':59\")  && (($9 + 0) == $9) && ($9 >= 200)) { print $9 }\' | sort | uniq -c | sort -n | tail'
			]
		}
	}

	var runner = new JobRunner(options);

	runner.on('consolidate', function(results){
		
		console.log(cli.green("################################################"));
		console.log(cli.green('Análise do HTTP Status Code - Varnish.'));
		console.log(cli.green('Ambiente: ' + env.toUpperCase()));
		console.log(cli.green('Componente: ' + component.toUpperCase()));
		console.log(cli.green('Data/Hora inicial:\t' +answers.startDate+'\t'+answers.startTime+':00'));
		console.log(cli.green('Data/Hora final:\t'+answers.endDate+'\t'+answers.endTime+':59'));
		console.log(cli.green("################################################"));

		var keys = results.keys();
		var origins = d3.map();
		var json = {};

		try{

			for (var i = keys.length - 1; i >= 0; i--) {
				
				var key = keys[i];
				var result = results.get(key);
				var raw = result[0].rawData;
				var values = raw.trim().split('\n');

				values.forEach(function(value){

					var sepValue = value.trim().split(' ');
					if(sKey != ''){
						var sKey = sepValue[1].replace('origin=','');
						var sVal = sepValue[0];

						if(!json[sKey]) {
							json[sKey] = [];
						}

						json[sKey].push({ server : key, value : sVal });

						if(origins.has(sKey)){
							origins.get(sKey).push(sVal);
						}else{
							origins.set(sKey,[sVal]);
						}
					}
				});

			};

			for (key in json) {
				var arr = json[key];
				for (var i = 0; i < keys.length; i++) {
					var server = keys[i];
					if(JSON.stringify(arr).indexOf(server) < 0) {
						json[key].push({ server: server, value: 0 });
					}
				}
			}

		}catch(err){
			console.log('Error: '+err);
		}

		var head =['#'];
		var aggregation = 0;

		// monta cabeçalho da tabela
		Object.keys(json).forEach(function(key) {
			var valueArray = json[key];
			valueArray.sort(function(a, b){
				return a.server.localeCompare(b.server)
			});
		    Object.keys(valueArray).forEach(function(i) {
		    	var value = valueArray[i].value;
		    	values.push(value);
		    	aggregation += parseInt(value);
		    	if(head.indexOf(valueArray[i].server) < 0) {
		    		head.push(valueArray[i].server);
		    	}
			});

		});

		head.push('TOTAL');
		head.push('%');
		var table = new Table({head:head, style : {compact : true, 'padding-left' : 1}});

		Object.keys(json).forEach(function(key) {
			
			var valueArray = json[key];

			valueArray.sort(function(a, b){
				return a.server.localeCompare(b.server)
			});
		    
		    var values = [];
		    var valuesNum = [];
		    var obj = {};
		    
		    Object.keys(valueArray).forEach(function(i) {
		    	var num = valueArray[i].value;
		    	valuesNum.push(num);
		    	values.push(numeral(num).format('0,0'));
			});

			var total = d3.sum(valuesNum);
			var percent = ((parseInt(total) / aggregation) * 100).toFixed(5);

			values.push(numeral(total).format('0,0'));
			values.push(percent);

			obj[key]=values;
			table.push(obj);

		});

		console.log(table.toString());

	});

runner.run();
});