var ChoicesFlow = require("../models/choicesFlow.js");
var Stage = require("../models/stage.js");
var JobRunner = require("../runner/jobRunner.js");
var flow = new ChoicesFlow();
var Table = require('cli-table');
var cli = require('cli-color');
var numeral = require('numeral');
var d3 = require('d3');
var Environment = require("../models/environment.js");

var date = new Date();
var timeFormated = date.toTimeString().split(" ")[0].substring(0, 5);
var dateArray = date.toDateString().split(" ");
var dateFormated = dateArray[2] + "/" + dateArray[1] + "/" + dateArray[3];

var stage = new Stage("Simple questions");

var env = "";
var component = "";

stage.addQuestion({
	type: "list",
	name: "env",
	message: "Inform the Environment",
	choices: [
		"prod",
		"stage",
		"qa",
		"dev"
	],
	default: 'prod',
	filter: function(answer) {
		env = answer;
		return env;
	}
});

stage.addQuestion({
	type: "list",
	name: "component",
	message: "Inform the components",
	choices: [
		"saas",
		"suggest",
		"solr_slaves",
		"solr_master"
	],
	default: 'saas',
	filter: function(answer) {
		component = answer;
		return component;
	}
});

stage.addQuestion({
	type: "checkbox",
	name: "servers",
	message: "Select servers",
	choices: function() {
		return Environment[env][component].servers;
	},
	default: function() {
		return Environment[env][component].servers;
	}
});

stage.addQuestion({
	type: "input",
	name: "size",
	message: "Results size",
	default: 5
});

stage.addQuestion({
	type: "input",
	name: "startDate",
	message: "Start date",
	default: dateFormated
});

stage.addQuestion({
	type: "input",
	name: "startTime",
	message: "Start time",
	default: "00:00"
});

stage.addQuestion({
	type: "input",
	name: "endDate",
	message: "End date",
	default: dateFormated
});

stage.addQuestion({
	type: "input",
	name: "endTime",
	message: "End time",
	default: timeFormated
});

stage.after(function(answers, callback) {
	callback();
});

flow.registerStage(stage);


flow.start(function(err, answers) {
	var authUtil = require('../utils/authUtil.js');
	var options = {
		servers: answers.servers,
		username: authUtil.getUsername(),
		password: authUtil.getPassword(),
		report: {
			name: 'QPS',
			commands: ['cat ' + Environment[env][component].accesslog + ' | grep GET | awk \'$4>"[' + answers.startDate + ':' + answers.startTime + ':00" && $4<"[' + answers.endDate + ':' + answers.endTime + ':59" {print substr($4, 14, 18)}\' | sort | uniq -c | sort -gr | head -' + answers.size]
		}
	}

	var runner = new JobRunner(options);

	runner.on('consolidate', function(results) {

		console.log(cli.green("################################################"));
		console.log(cli.green('QPS - Queries por Segundo'));
		console.log(cli.green('Ambiente: ' + env.toUpperCase()));
		console.log(cli.green('Componente: ' + component.toUpperCase()));
		console.log(cli.green('TOP ' + answers.size + ' QPS'));
		console.log(cli.green('Data/Hora inicial:\t' + answers.startDate + '\t' + answers.startTime + ':00'));
		console.log(cli.green('Data/Hora final:\t' + answers.endDate + '\t' + answers.endTime + ':59'));
		console.log(cli.green("################################################"));

		var keys = results.keys();
		var origins = d3.map();
		var json = {};

		try {

			for (var i = keys.length - 1; i >= 0; i--) {

				var key = keys[i];
				var result = results.get(key);
				var raw = result[0].rawData;
				var values = raw.trim().split('\n');
				var j = 1;

				values.forEach(function(value) {

					var sKey = j++;
					var sVal = value.replace(/\s+/, " ").trim();

					if (!json[sKey]) {
						json[sKey] = [];
					}

					json[sKey].push({
						server: key,
						value: sVal
					});

					if (origins.has(sKey)) {
						origins.get(sKey).push(sVal);
					} else {
						origins.set(sKey, [sVal]);
					}
				});

			};

		} catch (err) {
			console.log('Error: ' + err);
		}

		var head = ['#'];

		// monta cabeçalho da tabela
		Object.keys(json).forEach(function(key) {
			var valueArray = json[key];
			valueArray.sort(function(a, b) {
				return a.server.localeCompare(b.server)
			});
			Object.keys(valueArray).forEach(function(i) {
				var value = valueArray[i].value;
				values.push(value);
				if (head.indexOf(valueArray[i].server) < 0) {
					head.push(valueArray[i].server);
				}
			});

		});

		head.push('TOTAL');
		var table = new Table({
			head: head,
			style: {
				compact: true,
				'padding-left': 1
			}
		});

		Object.keys(json).forEach(function(key) {

			var valueArray = json[key];
			var total = 0;

			valueArray.sort(function(a, b) {
				return a.server.localeCompare(b.server)
			});

			var values = [];
			var obj = {};

			Object.keys(valueArray).forEach(function(i) {
				total += parseInt(valueArray[i].value.split(" ")[0]);
				values.push(valueArray[i].value);
			});

			values.push(total);

			obj[key] = values;
			table.push(obj);

		});

		console.log(table.toString());

	});

	runner.run();
});