var ChoicesFlow = require("../models/choicesFlow.js");
var Stage 		= require("../models/stage.js");
var JobRunner   = require("../runner/jobRunner.js");
var flow 		= new ChoicesFlow();
var d3 = require('d3');
var cli = require('cli-color');
var _ = require('underscore');
var Table = require('cli-table');
var moment = require('moment');
var tableUtil = require("../utils/tableUtil.js");

var stage = new Stage("Simple questions");
stage.addQuestion({
	type: "input",
	name: "servers",
	message: "Inform servers separated by comma",
	default: 'qa-supervisor-1,qa-supervisor-2,qa-supervisor-3,qa-supervisor-4,qa-supervisor-5'
});

stage.addQuestion({
	type: "input",
	name: "tracking",
	message: "Inform the tracking ID",
	default: "95c352be-0546-4d67-8082-34624e52dec9"
});

stage.after(function(answers, callback){
	var serversContent=answers.servers;
	answers.servers=serversContent.split(",");
	callback();
});
flow.registerStage(stage);

flow.start(function(err, answers){
	var authUtil = require('../utils/authUtil.js');
	var options={
		servers:answers.servers,
		username: authUtil.getUsername(),
		password: authUtil.getPassword(),
		report:{
			name:'First occurence',
			commands: [
				{name: 'First occurence', command:'grep \"'+answers.tracking+'\" /var/log/storm/worker-*.log | awk \'{print $1 \" \" $2}\'| head -1'},
				{name: 'Last occurence', command:'grep \"'+answers.tracking+'\" /var/log/storm/worker-*.log | awk \'{print $1 \" \" $2}\'| tail -1'},
				{name: 'Top events', command:'cat /var/log/storm/worker*.log  | grep c.w.s.h.s.RabbitEventSpout |  grep SUCCESS | awk \'{print substr($2, 1, 8)}\' | sort | uniq -c | sort -gr | head -15'},
				{name: 'Total received events', command: 'cat /var/log/storm/worker-*.log | grep RabbitConsumer |  grep \"Received\" | grep \"'+answers.tracking+'\" | wc -l'}
			]
		}
	}

	var runner = new JobRunner(options);
	runner.on('consolidate', function(results){
		console.log(cli.green("######################################"));
		console.log(cli.green('Analise HEIMDALL'));
		console.log(cli.green("######################################"));

		var lasts=[];
		var firsts=[];
		results.forEach(function(key, value){
			value.forEach(function(command){
				if(command.command.name==='First occurence'){
					firsts.push(command.rawData+"");
				}

				if(command.command.name==='Last occurence'){
					lasts.push(command.rawData+"");
				}
			});
		});

		//Formatting
		lasts = lasts.map(function(last){
			last = handleData(last);
			return last;
		});

		firsts = firsts.map(function(first){
			first = handleData(first);
			return first;
		});

		var last=moment(d3.max(lasts));
		var first=moment(d3.min(firsts));

		last=moment(last);
		first=moment(first);
		tableUtil.simpleTableFromResults(results, 'Top events');
		tableUtil.simpleTableFromResults(results, 'Total received events');

		var duration = moment.duration(last.diff(first));
		var hours = duration.asMinutes();

		var duration_table = new Table();
		duration_table.push(
			{ "Started": first.format()},
			{ "Finished": last.format()},
			{ "Total": hours + " minutes"}
		);
		console.log(duration_table.toString());
	});

runner.run();
});

function handleData(data){
	data=cleanData(data);
	data=convertToDate(data)
	return data;
}

function cleanData(data){
	var index = data.indexOf(".log:");
	if(index >= 0){
		data=data.substring(index+5, data.length);
	}
	data = data.replace(/\n/g,'');
	return data;
}

function convertToDate(data){
	var date = moment(data);
	return date.toDate().getTime();
}