var ChoicesFlow = require("../models/choicesFlow.js");
var Stage 		= require("../models/stage.js");
var JobRunner   = require("../runner/jobRunner.js");
var flow 		= new ChoicesFlow();
var d3 = require('d3');
var cli = require('cli-color');
var _ = require('underscore');
var Table = require('cli-table');
var numeral = require('numeral');
var Environment = require("../models/environment.js");

var stage = new Stage("Simple questions");

/* formating date */
var date = new Date();
var year = date.getFullYear();
var month = (date.getMonth() + 1);
var day = date.getDate();
var hour = date.getHours();
var minutes = date.getMinutes();
var dateFormated = (year + "-" + (month < 10 ? ("0" + month) : month) + "-" + (day < 10 ? ("0"+day) : day));
var timeFormated = ((hour < 10 ? ("0"+hour) : hour) + ":" + (minutes < 10 ? ("0"+minutes) : minutes));

var stage = new Stage("Simple questions");
var env = "";
var component = "";

stage.addQuestion({
	type: "list",
	name: "env",
	message: "Inform the Environment",
	choices: [
		"prod",
		"stage",
		"qa",
		"dev"
	],
	default: 'prod',
	filter: function(answer) {
		env = answer;
		return env;
	}
});

stage.addQuestion({
	type: "list",
	name: "component",
	message: "Inform the components",
	choices: [
		"indexer"
	],
	default: 'indexer',
	filter: function(answer) {
		component = answer;
		return component;
	}
});

stage.addQuestion({
	type: "checkbox",
	name: "servers",
	message: "Select servers",
	choices : function() {
		return Environment[env][component].servers;
	},
	default: function() {
		return Environment[env][component].servers;
	}
});

stage.addQuestion({
	type: "input",
	name: "startTime",
	message: "Start time",
	default: "00:00"
});

stage.addQuestion({
	type: "input",
	name: "endTime",
	message: "End time",
	default: timeFormated
});

stage.after(function(answers, callback){
	callback();
});
flow.registerStage(stage);

flow.start(function(err, answers){
	//console.log(answers);
	var commands=[];
	commands.push({name: 'indexed', command: 'cat ' + Environment[env][component].applog + ' | grep  ' + dateFormated + ' | grep " - Documents were indexed" | awk \'$2 > "\'' + answers.startTime + ':00\'" && $2 <= "\'' + answers.endTime + ':59\'" { SUM += $10 } END { if (SUM > 0) { print SUM } else { print "0" }}\''});
	commands.push({name: 'updated', command: 'cat ' + Environment[env][component].applog + ' | grep  ' + dateFormated + ' | grep " - Documents were updated" | awk \'$2 > "\'' + answers.startTime + ':00\'" && $2 <= "\'' + answers.endTime + ':59\'" { SUM += $10 } END { if (SUM > 0) { print SUM } else { print "0" }}\''});
	commands.push({name: 'removed', command: 'cat ' + Environment[env][component].applog + ' | grep  ' + dateFormated + ' | grep " - Documents were removed" | awk \'$2 > "\'' + answers.startTime + ':00\'" && $2 <= "\'' + answers.endTime + ':59\'" { SUM += $10 } END { if (SUM > 0) { print SUM } else { print "0" }}\''});

	var authUtil = require('../utils/authUtil.js');	
	
	var options={
		servers:answers.servers,
		username: authUtil.getUsername(),
		password: authUtil.getPassword(),
		report:{
			name:'QPS',
			commands: commands
		}
	}

	var runner = new JobRunner(options);
	var json = {};

	runner.on('consolidate', function(results){

		console.log(cli.green("####################################################"));
		console.log(cli.green('Consolidação dos documentos processados via Indexer.'));
		console.log(cli.green('Ambiente:\t' + env.toUpperCase()));
		console.log(cli.green('Componente:\t' + component.toUpperCase()));
		console.log(cli.green('Data:\t\t' + dateFormated));
		console.log(cli.green('Hora inicial:\t' + answers.startTime+':00'));
		console.log(cli.green('Hora final:\t'+ answers.endTime+':59'));
		console.log(cli.green("####################################################"));
		
		var agg = d3.map();
		var keys = results.keys();

		keys.forEach(function(server){
			var result = results.get(server);

			result.forEach(function(resul){
				var val = resul.rawData;
				val = val.trim().replace(/\n/g);

				if(!json[resul.command.name]) {
					json[resul.command.name] = [];
				}
				json[resul.command.name].push({server: server, value: val});
			});
		});

		var head =['#'];
		var aggregation = 0;

		// monta cabeçalho da tabela
		Object.keys(json).forEach(function(key) {
			var valueArray = json[key];
			valueArray.sort(function(a, b){
				return a.server.localeCompare(b.server)
			});
		    Object.keys(valueArray).forEach(function(i) {
		    	var value = valueArray[i].value;
		    	aggregation += parseInt(value);
		    	if(head.indexOf(valueArray[i].server) < 0) {
		    		head.push(valueArray[i].server);
		    	}
			});

		});

		head.push('TOTAL');
		head.push('%');
		var table = new Table({head:head, style : {compact : true, 'padding-left' : 1}});
		var count = 0;
		var array = [];
		var aux = [];
		var percentAux = 0;
		var valueAux = 0;
		var serverJson = {};

		Object.keys(json).forEach(function(key) {
			
			var valueArray = json[key];

			valueArray.sort(function(a, b){
				return a.server.localeCompare(b.server)
			});
		    
		    var values = [];
		    var valuesNum = [];
		    var obj = {};
		    
		    Object.keys(valueArray).forEach(function(i) {
		    	var num = valueArray[i].value;
		    	valuesNum.push(num);
		    	values.push(numeral(num).format('0,0'));

		    	if(!serverJson[valueArray[i].server]) {
		    		serverJson[valueArray[i].server] = 0;
		    	}
		    	serverJson[valueArray[i].server] += parseInt(num);

			});

			var total = d3.sum(valuesNum);
			var percent = ((parseInt(total) / aggregation) * 100).toFixed(5);

			values.push(numeral(total).format('0,0'));
			values.push(percent);

			valueAux += total;
			percentAux += parseFloat(percent);

			obj[key]=values;
			table.push(obj);

		});

		var obj2 = {};
		
		Object.keys(serverJson).forEach(function(value) {
			aux.push(numeral(serverJson[value]).format('0,0'));
		});

		aux.push(numeral(valueAux).format('0,0'));
		aux.push(percentAux);

		obj2['TOTAL'] = aux;
		table.push(obj2);

		console.log(table.toString());

	});

	runner.run();
});