var ChoicesFlow = require("../models/choicesFlow.js");
var Stage 		= require("../models/stage.js");
var JobRunner   = require("../runner/jobRunner.js");
var flow 		= new ChoicesFlow();
var d3 = require('d3');
var cli = require('cli-color');
var _ = require('underscore');
var Table = require('cli-table');
var numeral = require('numeral');
var Environment = require("../models/environment.js");

var stage = new Stage("Simple questions");

/* formating date */
var date = new Date();
var year = date.getFullYear();
var month = (date.getMonth() + 1);
var day = date.getDate();
var hour = date.getHours();
var minutes = date.getMinutes();
var dateFormated = (year + "-" + (month < 10 ? ("0" + month) : month) + "-" + (day < 10 ? ("0"+day) : day));
var timeFormated = ((hour < 10 ? ("0"+hour) : hour) + ":" + (minutes < 10 ? ("0"+minutes) : minutes));

var stage = new Stage("Simple questions");
var env = "";
var component = "";

stage.addQuestion({
	type: "list",
	name: "env",
	message: "Inform the Environment",
	choices: [
		"prod",
		"stage",
		"qa",
		"dev"
	],
	default: 'prod',
	filter: function(answer) {
		env = answer;
		return env;
	}
});

stage.addQuestion({
	type: "list",
	name: "component",
	message: "Inform the components",
	choices: [
		"indexer"
	],
	default: 'indexer',
	filter: function(answer) {
		component = answer;
		return component;
	}
});

stage.addQuestion({
	type: "checkbox",
	name: "servers",
	message: "Select servers",
	choices : function() {
		return Environment[env][component].servers;
	},
	default: function() {
		return Environment[env][component].servers;
	}
});

stage.addQuestion({
	type: "input",
	name: "size",
	message: "Top results",
	default: 5
});

stage.addQuestion({
	type: "input",
	name: "startTime",
	message: "Start time",
	default: "00:00"
});

stage.addQuestion({
	type: "input",
	name: "endTime",
	message: "End time",
	default: timeFormated
});

stage.after(function(answers, callback){
	callback();
});
flow.registerStage(stage);

flow.start(function(err, answers){
	//console.log(answers);
	var commands=[];
	commands.push({name: 'top', command: 'cat ' + Environment[env][component].applog + ' | grep ' + dateFormated + ' | grep LoggingIsVisibleFalseDocumentStep | awk \'$2 > "\'' + answers.startTime + ':00\'" && $2 <= "\'' + answers.endTime + ':59\'" {print $6}\' | grep -v Step | sort | uniq -c | sort -n | tail -n ' + answers.size + ' | sort -nrk 1,1'});

	var authUtil = require('../utils/authUtil.js');	
	
	var options={
		servers:answers.servers,
		username: authUtil.getUsername(),
		password: authUtil.getPassword(),
		report:{
			name:'QPS',
			commands: commands
		}
	}

	var startProcess = new Date();

	var runner = new JobRunner(options);
	var json = {};

	runner.on('consolidate', function(results){

		var keys = results.keys();
		var origins = d3.map();
		var json = {};

		try{

			for (var i = keys.length - 1; i >= 0; i--) {
				
				var key = keys[i];
				var result = results.get(key);
				var raw = result[0].rawData;
				var values = raw.trim().split('\n');
				var j = 1;

				values.forEach(function(value){

					var sKey = j++;
					var sVal = value.replace(/\s+/, " ").trim();

					if(!json[sKey]) {
						json[sKey] = [];
					}

					json[sKey].push({ server : key, value : sVal });

				});

			};

			for (key in json) {
				var arr = json[key];
				for (var i = 0; i < keys.length; i++) {
					var server = keys[i];
					if(JSON.stringify(arr).indexOf(server) < 0) {
						json[key].push({ server: server, value: 0 });
					}
				}
			}

		}catch(err){
			console.log('Error: '+err);
		}

		var head =['#'];

		// monta cabeçalho da tabela
		Object.keys(json).forEach(function(key) {
			var valueArray = json[key];
			valueArray.sort(function(a, b){
				return a.server.localeCompare(b.server)
			});
		    Object.keys(valueArray).forEach(function(i) {
		    	var value = valueArray[i].value;
		    	values.push(value);
		    	if(head.indexOf(valueArray[i].server) < 0) {
		    		head.push(valueArray[i].server);
		    	}
			});

		});

		head.push('TOTAL');
		var table = new Table({head:head, style : {compact : true, 'padding-left' : 1}});

		Object.keys(json).forEach(function(key) {
			
			var valueArray = json[key];
			var total = 0;

			valueArray.sort(function(a, b){
				return a.server.localeCompare(b.server)
			});
		    
		    var values = [];
		    var obj = {};
		    
		    Object.keys(valueArray).forEach(function(i) {
		    	total += parseInt(valueArray[i].value.toString().split(" ")[0]);
		    	values.push(valueArray[i].value);
			});

		    values.push(total);

			obj[key]=values;
			table.push(obj);

		});

		var endProcess = new Date();
		var timeDiff = Math.abs(endProcess.getTime() - startProcess.getTime());

		console.log(cli.green("############################################"));
		console.log(cli.green('TOP ' + answers.size + ' documentos processados no indexador.'));
		console.log(cli.green('Ambiente:\t' + env.toUpperCase()));
		console.log(cli.green('Componente:\t' + component.toUpperCase()));
		console.log(cli.green('Data:\t\t' + dateFormated));
		console.log(cli.green('Hora inicial:\t' + answers.startTime+':00'));
		console.log(cli.green('Hora final:\t'+ answers.endTime+':59'));
		console.log(cli.green('Tempo total:\t'+ (timeDiff / 1000) +' seconds.'));
		console.log(cli.green("############################################"));

		console.log(table.toString());

	});

	runner.run();
});