var ChoicesFlow = require("../models/choicesFlow.js");
var Stage 		= require("../models/stage.js");
var JobRunner   = require("../runner/jobRunner.js");
var flow 		= new ChoicesFlow();
var d3 = require('d3');
var cli = require('cli-color');
var _ = require('underscore');
var Table = require('cli-table');

var stage = new Stage("Simple questions");
stage.addQuestion({
	type: "input",
	name: "servers",
	message: "Inform servers sepated by comma",
	default: 'prod-saas-1,prod-saas-2,prod-saas-3,prod-saas-4'
});

stage.after(function(answers, callback){
	var serversContent=answers.servers;
	answers.servers=serversContent.split(",");
	callback();
});
flow.registerStage(stage);

flow.start(function(err, answers){
	//console.log(answers);
	var commands=[];
	commands.push({name: 'Other errors', command: 'cat /walmart/log/ws-search-service/search.log | grep requestLogger | grep fail | grep -v \"invalid parameter using solr\" | wc -l '});
	commands.push({name: 'Invalid parameter using solr', command: 'cat /walmart/log/ws-search-service/search.log | grep requestLogger | grep fail | grep "invalid parameter using solr" | wc -l '});
	commands.push({name: 'Parameters is not valid', command: 'cat /walmart/log/ws-search-service/search.log | grep requestLogger | grep fail | grep "parameters is not valid" | wc -l '});
	commands.push({name: 'Total Requests', command: 'cat /walmart/log/ws-search-service/search.log | grep requestLogger | wc -l'});

	var authUtil = require('../utils/authUtil.js');	
	
	var options={
		servers:answers.servers,
		username: authUtil.getUsername(),
		password: authUtil.getPassword(),
		report:{
			name:'QPS',
			commands: commands
		}
	}

	var runner = new JobRunner(options);

	runner.on('consolidate', function(results){
		var agg = d3.map();
		results.keys().forEach(function(server){
			var result = results.get(server);
			result.forEach(function(resul){
				var val = resul.rawData;
				val = val.trim().replace(/\n/g);
				if(!agg.has(resul.command.name)){
					agg.set(resul.command.name, parseInt(val));
				}else{
					var value = agg.get(resul.command.name);
					value+=parseInt(val);
					agg.set(resul.command.name, value);
				}
			});
		});

		/* cross */
		var headers=['#', 'count', '%']
		var cross_table = new Table({ head: headers});
		var total = agg.get('Total Requests');
		agg.forEach(function(key, val){
			varArr=[];
			varArr.push(val);
			varArr.push((val/total * 100) + '%');
			this.set(key,varArr);
		});

		agg.forEach(function(key, val){
			var row={};
			row[key]=val;
			cross_table.push(row);
		});
		console.log(cross_table.toString());
	});

	runner.run();
});