var ChoicesFlow = require("../models/choicesFlow.js");
var Stage 		= require("../models/stage.js");
var JobRunner   = require("../runner/jobRunner.js");
var flow 		= new ChoicesFlow();
var d3 = require('d3');
var cli = require('cli-color');
var _ = require('underscore');

var stage = new Stage("Simple questions");
stage.addQuestion({
	type: "input",
	name: "servers",
	message: "Inform servers sepated by comma",
	default: 'prod-saas-1,prod-saas-2,prod-saas-3,prod-saas-4'
});

stage.addQuestion({
	type: "input",
	name: "statusCode",
	message: "Status Code Error",
	default: '500'
});

stage.addQuestion({
	type: "input",
	name: "size",
	message: "Quantity of rows",
	default: '1'
});

stage.after(function(answers, callback){
	var serversContent = answers.servers;
	answers.servers = serversContent.split(",");
	callback();
});
flow.registerStage(stage);

flow.start(function(err, answers){
	//console.log(answers);
	var authUtil = require('../utils/authUtil.js');
	var options={
		servers:  answers.servers,
		username: authUtil.getUsername(),
		password: authUtil.getPassword(),
		report:{
			name:'QPS',
			commands: [
				'awk \'($10 ~ /' + answers.statusCode + '/)\' /var/log/nginx/access-saas.log | tail -n ' + answers.size
			]
		}
	}

	var runner = new JobRunner(options);

	runner.on('consolidate', function(results){
		console.log(cli.green("################################################"));
		console.log(cli.green("Análise do HTTP Status Code Error: " + answers.statusCode + " size: " + answers.size));
		console.log(cli.green("################################################"));

		var keys = results.keys();
		var originValues = d3.map();

		try{
			for (var i = keys.length - 1; i >= 0; i--) {
				var key = keys[i];
				var result = results.get(key);
				console.log(cli.yellow("Result for: " + key));
				console.log(result[0].rawData);
			};

		}catch(err){
			console.log('Error: '+err);
		}
	});

runner.run();
});