var fs = require('fs');
var path = require('path');
var ChoicesFlow = require("../models/choicesFlow.js");
var Stage 		= require("../models/stage.js");
var JobRunner   = require("../runner/jobRunner.js");
var flow 		= new ChoicesFlow();
var d3 = require('d3');
var cli = require('cli-color');
var _ = require('underscore');

var stage = new Stage("Simple questions");
stage.addQuestion({
	type: "input",
	name: "servers",
	message: "Inform servers sepated by comma",
	default: 'prod-saas-1,prod-saas-2,prod-saas-3,prod-saas-4'
});

stage.addQuestion({
	type: "input",
	name: "time",
	message: "Inform min time in miliseconds",
	default: '1000'
});

stage.addQuestion({
	type: "input",
	name: "start",
	message: "Start time",
	default: '12:00'
});

stage.addQuestion({
	type: "input",
	name: "end",
	message: "End time",
	default: '13:00'
});

stage.addQuestion({
	type: "input",
	name: "begin",
	message: "Begin",
	default: 'tail'
});

stage.addQuestion({
	type: "input",
	name: "size",
	message: "Quantity of rows",
	default: '1'
});

stage.addQuestion({
	type: "confirm",
	name: "export",
	message: "Do you want export data to .csv file?",
	default: 'false'
});

stage.after(function(answers, callback){
	var serversContent = answers.servers;
	answers.servers = serversContent.split(",");
	callback();
});
flow.registerStage(stage);

flow.start(function(err, answers){
	//console.log(answers);
	var authUtil = require('../utils/authUtil.js');
	var options={
		servers:  answers.servers,
		username: authUtil.getUsername(),
		password: authUtil.getPassword(),
		report:{
			name:'QPS',
			commands: [
				'cat /walmart/log/ws-search-service/search.log  | grep requestLogger | awk \'/' + answers.start + ':/,/' + answers.end + ':/  {print}\' | awk \'{if ((substr($NF, 2, index($NF, "]")-2)+0) > ' + answers.time + ') print}\' | ' + answers.begin + ' -n ' + answers.size + ' | awk \'{print substr($9, index($9,"{"), index($9, "}")) }\''
			]
		}
	}

	var runner = new JobRunner(options);

	runner.on('consolidate', function(results){
		console.log(cli.green("########################################################"));
		console.log(cli.green("Análise dos tempos de respostas: " + answers.time + " size: " + answers.size));
		console.log(cli.green("########################################################"));

		var keys = results.keys();
		var originValues = d3.map();

		var csvContent='';
		try{
			for (var i = keys.length - 1; i >= 0; i--) {
				var key = keys[i];
				var result = results.get(key);
				console.log(cli.yellow("Result for: " + key));
				console.log(result[0].rawData);
				csvContent+=result[0].rawData+',';
			};

		}catch(err){
			console.log('Error: '+err);
		}

		if(answers.export === true){
			var pathFile = path.join('.', 'data.csv');
			fs.writeFile(pathFile, csvContent, function(err) {
				if(err) {
					console.log(err);
				}
				console.log('   \033[36mcreate\033[0m : ' + pathFile);
			}); 
		}
	});



runner.run();
});