var ChoicesFlow = require("../models/choicesFlow.js");
var Stage 		= require("../models/stage.js");
var JobRunner   = require("../runner/jobRunner.js");
var flow 		= new ChoicesFlow();
var d3 = require('d3');
var cli = require('cli-color');
var _ = require('underscore');
var Table = require('cli-table');

/* formating date */
var date = new Date();
var year = date.getFullYear();
var month = (date.getMonth() + 1);
var day = date.getDate();
var hour = date.getHours();
var minutes = date.getMinutes();
var dateFormated = (year + "-" + (month < 10 ? ("0" + month) : month) + "-" + (day < 10 ? ("0"+day) : day));
var timeFormated = ((hour < 10 ? ("0"+hour) : hour) + ":" + (minutes < 10 ? ("0"+minutes) : minutes));

var stage = new Stage("Simple questions");
stage.addQuestion({
	type: "input",
	name: "servers",
	message: "Inform servers separated by comma",
	default: 'prod-saas-1,prod-saas-2,prod-saas-3,prod-saas-4'
});

stage.addQuestion({
	type: "input",
	name: "startTime",
	message: "Start time",
	default: "00:00"
});

stage.addQuestion({
	type: "input",
	name: "endTime",
	message: "End time",
	default: timeFormated
});

stage.after(function(answers, callback){
	var serversContent=answers.servers;
	answers.servers=serversContent.split(",");
	callback();
});
flow.registerStage(stage);

flow.start(function(err, answers){
	var authUtil = require('../utils/authUtil.js');
	var options={
		servers:answers.servers,
		username: authUtil.getUsername(),
		password: authUtil.getPassword(),
		report:{
			name:'QPS',
			commands: ['cat /walmart/log/ws-search-service/search.log* | grep  ' + dateFormated + ' | awk \'$2 > "' + answers.startTime + '" && $2 <= "' + answers.endTime + '" { print }\' | awk \'{print $9}\' | awk \'{split($0, arr, "&"); for(queryString in arr) print arr[queryString] }\' | grep "^origin=*" | sort | uniq -c | sort']
		}
	}

	var runner = new JobRunner(options);

	runner.on('consolidate', function(results){
		console.log(cli.green("######################################"));
		console.log(cli.green('Analise de origin dos requests do SAAS'));
		console.log(cli.green("######################################"));
		var keys = results.keys();
		var origins = d3.map();
		try{
			for (var i = keys.length - 1; i >= 0; i--) {
				var key = keys[i];
				var result = results.get(key);
				var raw = result[0].rawData;
				var values = raw.trim().split('\n');
				values.forEach(function(value){
					var sepValue = value.trim().split(' ');
					if(sKey != ''){
						var sKey = sepValue[1].replace('origin=','');
						var sVal = sepValue[0];
						if(origins.has(sKey)){
							origins.get(sKey).push(sVal);
						}else{
							origins.set(sKey,[sVal]);
						}
					}
				});
			};
		}catch(err){
			console.log('Error: '+err);
		}

		var head =['#'];
		head = head.concat(keys);
		head.push('TOTAL');
		var table = new Table({head:head, style : {compact : true, 'padding-left' : 1}});
		origins.forEach(function(key, value){
			var obj={};
			var total = d3.sum(value);
			value.push(total);
			obj[key]=value;
			table.push(obj);	
		});
		console.log(table.toString());
	});

runner.run();
});
