var ChoicesFlow = require("../models/choicesFlow.js");
var Stage 		= require("../models/stage.js");
var JobRunner   = require("../runner/jobRunner.js");
var flow 		= new ChoicesFlow();
var d3 = require('d3');
var cli = require('cli-color');
var _ = require('underscore');
var Table = require('cli-table');


var stage = new Stage("Simple questions");

stage.addQuestion({
	type: "input",
	name: "servers",
	message: "Inform servers sepated by comma",
	default: 'prod-saas-1,prod-saas-2,prod-saas-3,prod-saas-4'
});

stage.addQuestion({
	type: "input",
	name: "size",
	message: "Results size",
	default: 5
});

stage.after(function(answers, callback){
	var serversContent=answers.servers;
	answers.servers=serversContent.split(",");
	answers.batteries=[
		{name:'Bateria 1', start:'00:07', end: '00:57'},
		{name:'Bateria 2', start:'01:31', end: '02:31'},
		{name:'Bateria 3', start:'03:25', end: '04:40'},
		{name:'Bateria 4', start:'04:41', end: '06:00'}
	];

	callback();
});

flow.registerStage(stage);


flow.start(function(err, answers){
	var commands=[];
	answers.batteries.forEach(function(battery){
		commands.push({name: battery.name + ' [start:'+battery.start+', end:'+battery.end+"]", command:'cat /walmart/log/ws-search-service/search.log  | grep requestLogger  | awk \' /' + battery.start + ':/,/' + battery.end + ':/ {print substr($2, 1, 8)}\' | sort | uniq -c | sort -gr | head -'+answers.size});
	});
	var authUtil = require('../utils/authUtil.js');
	var options={
		servers:answers.servers,
		username: authUtil.getUsername(),
		password: authUtil.getPassword(),
		report:{
			name:'QPS',
			commands:commands
		}
	}

	var runner = new JobRunner(options);

	runner.on('consolidate', function(results){
		results.keys().forEach(function(server){
			var result = results.get(server);
			console.log(cli.green("######################################"));
			console.log(cli.green(''+server));
			console.log(cli.green("######################################"));
			//console.log(result);
			var headers = result.map(function(res){
				//console.log(res);
				return res.command.name;
			});

			var values = result.map(function(res){
				return res.rawData;
			});

			var head =[];
			head = head.concat(headers);
			var table = new Table({head:head, style : {compact : true, 'padding-left' : 1}});
			table.push(values);
			console.log(table.toString());
		});
	});

	runner.run();
});
