var ChoicesFlow = require("../models/choicesFlow.js");
var Stage 		= require("../models/stage.js");
var JobRunner   = require("../runner/jobRunner.js");
var flow 		= new ChoicesFlow();
var d3 = require('d3');
var cli = require('cli-color');
var _ = require('underscore');
var Table = require('cli-table');

var stage = new Stage("Simple questions");
stage.addQuestion({
	type: "input",
	name: "servers",
	message: "Inform servers sepated by comma",
	default: 'prod-saas-1,prod-saas-2,prod-saas-3,prod-saas-4'
});

stage.after(function(answers, callback){
	var serversContent=answers.servers;
	answers.servers=serversContent.split(",");
	callback();
});
flow.registerStage(stage);

flow.start(function(err, answers){
	//console.log(answers);
	var authUtil = require('../utils/authUtil.js');
	var options={
		servers:answers.servers,
		username: authUtil.getUsername(),
		password: authUtil.getPassword(),
		report:{
			name:'QPS',
			commands: ['cat /walmart/log/ws-search-service/search.log | grep requestLogger | wc -l']
		}
	}

	var runner = new JobRunner(options);

	runner.on('consolidate', function(results){
		console.log(cli.green("######################################"));
		console.log(cli.green('Analise de requests do SAAS'));
		console.log(cli.green("######################################"));

		var keys = results.keys();
		var originValues = [];
		var head = keys;
		
		try{
			for (var i = keys.length - 1; i >= 0; i--) {
				var key = keys[i];
				var result = results.get(key);
				//console.log('Result for: '+key);
				var values = result[0].rawData.trim().split('\n');
				//console.log(result[0].rawData+"");
				originValues.push(result[0].rawData+"".trim());
			};
		}catch(err){
			console.log('Error: '+err);
		}
		head.push('Total')
		var table = new Table({head:head, style : {compact : true, 'padding-left' : 1}});
		var total = d3.sum(originValues);
		originValues.push(total);
		table.push(originValues);
		console.log(table.toString());
	});

runner.run();
});