var fs = require("fs");
var path = require('path'); 
var commandDir = path.join(process.cwd());
var winston = require('winston');

var Environment = function(properties){
	if(!properties){
		throw "Properties environment cannot be null";
	}
	return {
		raw: properties,
		getEnvironmentList: function(){
			var environments=[];
			for(env in properties.environment){
				environments.push(env);
			}
			return environments;
		},
		getComponentList: function(environment){
			winston.debug('getComponentList: ', environment);
			var components = [];
			for(component in this.raw.environment[environment]){
				components.push(component);
			}
			return components;
		},
		getComponent: function(environmentName, componentName){
			winston.debug('GetComponent: ', environmentName, componentName);
			return this.raw.environment[environmentName][componentName];
		},
		getCommands: function(componentName){
			//TODO: FILTRAR ARRAY E RETORNAR SOMENTE O NOME DO COMANDO
			return this.raw.commands[componentName];
		},
		//Return all commands configurated in sherlock.json independently from component
		getAllCommands: function(componentName){
			return this.raw.commands;
		},
		getCommand: function(componentName, commandName){
			var cmd = null;
			this.raw.commands[componentName].forEach(function(command){
				if(command.name == commandName){
					var commandFunction = require(commandDir+"/"+command.file);
					cmd = {name: commandName, command: commandFunction, params: command.params}	
				}
			});
			return cmd;
		}
	}
}

var exists = fs.existsSync('./sherlock.json');
if(exists){
	var sherlockConfigContent = fs.readFileSync('./sherlock.json');
	var sherlockConfig = JSON.parse(sherlockConfigContent);
	module.exports = new Environment(sherlockConfig);
}else{
	var configurationStage = require("../stages/configurationStage.js").configurationStage;
	var Flow = require('./choicesFlow');
	configFlow = new Flow();
	configFlow.registerStage(configurationStage);
	configFlow.start(function(err, answers){
			var sherlockConfigContent = fs.readFileSync('./sherlock.json');
			var sherlockConfig = JSON.parse(sherlockConfigContent);
			module.exports = new Environment(sherlockConfig);
	});
}