var inquirer = require('inquirer');
var Sync = require("sync");
var async = require("async");
var winston = require('winston');
var merge = require('merge');

var Stage = function Stage(name, properties){
	return {
		name:name,
		apply:function(){
			return true;
		},
		questions:[],
		beforeFN: null,
		afterFN: function(answers, callback){
			callback();
		},
		before:function(fn){
			this.beforeFN=fn;
		},
		after:function(fn){
			this.afterFN=fn;
		},
		registerController: function(stageController){
			this.controller = stageController;
		},
		applyWhen: function(applyFunction){
			this.apply=applyFunction;
		},
		addQuestion: function(question){
			if(!isFunction(question)){
				this.questions.push(function(answers){
					return question;
				});	
			}else{
				this.questions.push(question);
			}
		},
		shouldExecuteStage: function(previousAnswers){
			return this.apply!= null && this.apply(previousAnswers);
		},
		execute: function(previousAnswers, callback){
			var stage = this;
			var callbackToReturn= (callback != undefined)? callback: previousAnswers;

			//Verifying if should execute
			if(!this.shouldExecuteStage(previousAnswers)){
				callbackToReturn(null, previousAnswers);
				return;
			} 

			response={};	
			if(previousAnswers){
				response = mergeJson(previousAnswers, response);
			}
			if(response == null || response == undefined){
				response={};
				for(prop in properties){
					response[prop]=properties[prop];
				}	
			}

			winston.debug('Properties: ', properties);
			winston.debug('Response: ', response);

			//Executing questions
			async.eachSeries(this.questions, executeInquirer, function(err){
				if(stage.afterFN!=null){
					Sync(function(){
						stage.afterFN(response, function(err){
							callbackToReturn(null, response);
						});
					});
					return;
				}

				callbackToReturn(null, response);
				response=null;
			});	
		}
	}
};

module.exports =Stage;

var response=null;
function executeInquirer(question, callback){
	winston.debug("executeInquirer: ", response);
	var extractedQuestion = question(response);
	if(extractedQuestion.apply && !extractedQuestion.apply(response)){
		callback();
	}else{
		inquirer.prompt([extractedQuestion], function(answers){
			var mergedResponse = mergeJson(response, answers);
			response = mergedResponse;
			callback();
		});
	}
}

function isFunction(functionToCheck) {
 var getType = {};
 return functionToCheck && getType.toString.call(functionToCheck) === '[object Function]';
}

function isEmpty(obj) {
    return Object.keys(obj).length === 0;
}

function mergeJson(olderJson, newerJson){
	winston.debug("##### MERGE JSON ####### ", olderJson, newerJson);
	if(olderJson == null || olderJson == 'undefined' || olderJson == undefined || isFunction(olderJson) || isEmpty(olderJson)){
		return newerJson;
	}

	if(isEmpty(newerJson)) {
		return olderJson;
	}
	var mergedResponse = merge(olderJson, newerJson);
	winston.debug("Merged response: ", mergedResponse);
	return mergedResponse;
}