var util         = require("util");
var EventEmitter = require("events").EventEmitter;
var SingleRunner = require('./singleRunner.js');
var d3 = require('d3');

var JobRunner = function(options){
	this.options=options;
	this.runners=[];
	this.results=d3.map();
	this.jobCounter=0;
	var instance=this;

	instance.options.servers.forEach(function(server){
		var options2SingleRunner={
			server:server,
			sshPort: instance.options.sshPort,
			username:instance.options.username,
			password:instance.options.password,
			report:instance.options.report,
			stream: options.stream
		}
		var singleRunner = new SingleRunner(options2SingleRunner);

		singleRunner.on('data', function(result){
			instance.emit('data', result);
		});

		singleRunner.on('end', function(results){
			instance.jobCounter++;
			instance.results.set(singleRunner.options.server, results);
			instance.emit('execution', {percent: (instance.jobCounter/instance.runners.length * 100), message: singleRunner.options.server});
			if(instance.jobCounter==instance.runners.length){
				instance.emit('consolidate', instance.results);
			}
		});
		instance.runners.push(singleRunner);
	});

	instance.run=function(){
		instance.emit('start');
		instance.runners.forEach(function(runner){
			runner.run();
		});
	}

	//Configuring EventEmitter
	EventEmitter.call(this);
}

util.inherits(JobRunner, EventEmitter);
module.exports=JobRunner;