var util         = require("util");
var EventEmitter = require("events").EventEmitter;
var Connection   = require('ssh2');
var cli 		 = require('cli-color');
var winston = require('winston');

var SingleRunner = function(options){
	winston.debug("Options: ", options);
	this.options=options;
	this.results=[];
	this.executionCounter=0;
	var instance=this;
	instance.run=function(){
		var conn = new Connection();
		conn.on('ready', function() {
			instance.on('end', function(){
				conn.end();
			});
		  winston.debug('Connection :: ready');
		  options.report.commands.forEach(function(command){
		  	winston.debug('Executing command: '+command.name);
		  	var shellCommand = '';
		  	if(command!= null && typeof command === 'object'){
		  		shellCommand = command.command;
		  	}else{
		  		shellCommand=command;
		  	}
		  	instance.emit('start', 'Running in the server: '+options.server+ ", command: "+shellCommand);
		  	winston.debug("Running command: ", shellCommand);
		  	var received = false;
		  	conn.exec(shellCommand, function(err, stream){
				if (err) throw err;
				stream.on('exit', function(code, signal) {
					winston.debug('Stream :: exit :: code: ' + code + ', signal: ' + signal);
				}).on('close', function() {
					winston.debug('Stream :: close');
					if(!received){
						var dataResult = {
							command: command,
							server: instance.options.server,
							rawData: "EMPTY"
						}
						instance.emit('data', dataResult);
					}
				}).on('data', function(data) {
					winston.debug("Receiving data: ", data);
					var dataResult = {
						command: command,
						server: instance.options.server,
						rawData: data+""
					}
					received=true;
					instance.emit('data', dataResult);
				}).stderr.on('data', function(data) {
					console.log(cli.yellow('STDERR: ' + data));
				});
		  	});
		  });
		}).connect({
		  host: options.server,
		  port: options.sshPort || 22,
		  username: options.username,
		  password: options.password,
		  readyTimeout: 120000 // in milliseconds
		});
	}

	instance.on('data', function(dataResult){
		instance.results.push(dataResult);
		instance.executionCounter++;
		if(instance.executionCounter == instance.options.report.commands.length && !options.stream==true){
			winston.debug("Emiting 'end' event in SingleRunner");
			instance.emit('end', instance.results);
		}
	});

	EventEmitter.call(this);
}

util.inherits(SingleRunner, EventEmitter);
module.exports=SingleRunner;