var Stage = require("../models/stage.js");
var fs = require("fs");
var path = require("path");
var mkdirp = require('mkdirp');
var async = require('async');
var sherlockFile = path.join('.', 'sherlock.json');
var config={};

//###########################################
//###### Configuring Stage
//###########################################
var configurationStage = new Stage("Configuration stage");
configurationStage.applyWhen(function(answers){
	return !fs.existsSync("sherlock.json");
});

configurationStage.addQuestion({
	type: "confirm",
	name: "createProject",
	message: "Do you want create a Sherlock project here?",
	default: false
});

configurationStage.addQuestion({
	type: "confirm",
	name: "sampleProject",
	message: "Do you want sample jobs?",
	default: false
});

configurationStage.after(function(answers, callback){
	if(answers.createProject){
		var files =['jobs','commands','visualizations'];
		config.folders=files;
		config.environment = {
			'prod': {
				'appA': {
					servers: ['prod-appA-1','prod-appA-2','prod-appA-3','prod-appA-4'],
					acessLog: '/var/log/nginx/access-appA.log*',
					applog: '/walmart/log/ws-search-service/search.log'
				},
				'appB':{
					servers: ['prod-appB-1','prod-appB-2','prod-appB-3','prod-appB-4'],
					acessLog: '/var/log/nginx/access-appB.log*',
					applog: '/walmart/log/ws-search-appB/search.log'
				}
			},
			'qa': {
				'appA': {
					servers: ['qa-appA-1','qa-appA-2','qa-appA-3','qa-appA-4'],
					acessLog: '/var/log/nginx/access-appA.log*',
					applog: '/walmart/log/ws-search-service/search.log'
				},
				'appB':{
					servers: ['qa-appB-1','qa-appB-2','qa-appB-3','qa-appB-4'],
					acessLog: '/var/log/nginx/access-appB.log*',
					applog: '/walmart/log/ws-search-appB/search.log'
				}
			}
		}
		async.eachSeries(files, mkdir, function(err){
			if(err){
				console.error("Problem to create dir: "+err);
			}else{
				sherlockFile = {
					path: path.join('.', 'sherlock.json'),
					content: JSON.stringify(config,null, "\t")
				}

				createConfigFiles([sherlockFile], function(err){
					callback(err);
				});
			}
		});
	}
});
//###########################################
//###### Configuring Stage
//###########################################
function createConfigFiles(files, callback){
	async.eachSeries(files, createComplexFile, function(err){
		callback(err);
	});
}

function createComplexFile(file, callback){
	createConfigFile(file.content, file.path, callback);
}

function createConfigFile(config, path, callback){
	fs.writeFile(path, config, function(err) {
		if(err) {
			console.log(err);
		}
		console.log('   \033[36mcreate\033[0m : ' + path);
		callback(err);
	});
}

function exist(file, callback){
	var result = fs.existsSync(file);
	return result;
}

function mkdir(dir, fn){
	directory =  path.join('.', dir);
	if(!exist(directory)){
		mkdirp(directory, 0755, function(err){
			if (err){
				throw err;
			}
			console.log('   \033[36mcreate\033[0m : ' + directory);
			fn();
		});
	}
}

module.exports.configurationStage = configurationStage;