require('coffee-script/register');

var fs = require('fs');

var chai = require('chai');
var expect = chai.expect;
var should = require('should');

var Context = require('../../../../app/helpers/executions/context.coffee');
var Flow = require('../../../../app/helpers/executions/flow.coffee');
var Step = require('../../../../app/helpers/executions/step.coffee');
var SSH = require('../../../../app/helpers/executions/ssh.coffee');
var SCP = require('../../../../app/helpers/executions/scp.coffee');
var Local = require('../../../../app/helpers/executions/local.coffee');
var Commands = require('../../../../app/helpers/executions/commands.coffee');


describe("Flow - spec", function(){
  it('should throw exception when try add step without step instance', function(){
    var flow = new Flow();
    should.exist(flow);
    try{
      flow.add({});
      false.should.be.true
    }catch(error){
       "add must receive a Step instance as parameter".should.be.equal(error);
    }
  })
  it('should add steps', function(){
    var flow = new Flow();
    should.exist(flow);
    flow.add(new Step());
    flow.steps.should.have.length(1);
  })

  it('should execute SSH step', function(done){
    var servers = ['127.0.0.1'];
    var sshPort = 2222;

    var context = new Context();
    context.ssh({
      commands: [{name:'Hello', command: 'echo "Hello Sherlock"'}]
    });

    var flow = new Flow(null, context);
    var ssh = new SSH(servers, sshPort);

    should.exist(flow);
    should.exist(ssh);

    flow.executeStep(ssh, function(err, results){
      done();
    })
  })

  it('should execute SCP step', function(done){
    this.timeout(10000);
    var servers = ['127.0.0.1'];
    var scpPort = 2222;

    var filename = 'test.txt'

    var context = new Context();
    context.scp({
      path:'/home/vagrant/'+filename,
      target: 'trash/'
    });

    var flow = new Flow(null, context);
    var scp = new SCP(servers, scpPort);

    should.exist(flow);
    should.exist(scp);
    servers.should.be.equal(scp.servers);
    scpPort.should.be.equal(scp.scpPort);

    if(fs.existsSync(context.scpRequest.target+filename)){
      fs.unlinkSync(context.scpRequest.target+filename);
    }

    var exists = fs.existsSync(context.scpRequest.target+filename);
    exists.should.be.false;

    flow.executeStep(scp, function(err, results){
      var exists = fs.existsSync(context.scpRequest.target+filename);
      exists.should.be.true;
      done();
    })
  })

  it('should execute Local step', function(done){
    var filePath = 'trash/example.txt'

    var context = new Context();
    var commands = new Commands();
    commands.addCommand('Create '+filePath, 'echo "Hello" >> '+filePath);
    context.local(commands);

    var flow = new Flow(null, context);
    var local = new Local();

    if(fs.existsSync(filePath)){
      fs.unlinkSync(filePath);
    }

    var exists = fs.existsSync(filePath);
    exists.should.be.false;

    flow.executeStep(local, function(err, results){
      var exists = fs.existsSync(filePath);
      exists.should.be.true;
      done();
    })
  })

  it('should execute step by method execute', function(done){
      var context = new Context();
      var commands = new Commands();
      commands.addCommand('Uname', 'uname -a');
      context.local(commands);
      var local = new Local();

      var flow = new Flow(null, context);
      flow.add(local);
      flow.on('end_flow', function(err, results){
        done();
      })
      flow.execute();
  });

  it('should execute multiple steps by method execute', function(done){
      this.timeout(10000);
      var path = '/home/vagrant/'
      var filename = path+'sherlock.txt'
      var servers = ['127.0.0.1'];
      var sshPort = 2222;
      var context = new Context();
      var flow = new Flow(null, context);

      var commands = new Commands();
      commands.addCommand('Uname', 'uname -a');
      commands.addCommand('Pwd', 'pwd');
      commands.addCommand('List', 'ls -lart');

      context.local(commands);
      context.ssh({
        commands: [
          {name:'Hello', command: 'echo "Hello Sherlock"'},
          {name:'List file: '+path, command: 'ls -lart '+path}
        ]
      });
      context.scp({
        path:'/home/vagrant/'+filename,
        target: 'trash/'
      });

      var local = new Local();
      var ssh = new SSH(servers, sshPort);
      var scp = new SCP(servers, sshPort);

      flow.add(ssh);
      flow.add(scp);
      flow.add(local);

      flow.on('end_flow', function(err, results){
        console.log(JSON.stringify(results, null, '\t'));
        done();
      })
      flow.execute();
  });
})