require('coffee-script/register');

var chai = require('chai');
var expect = chai.expect;
var should = require('should');

describe("SCP - spec", function(){
  it('should throw message error when servers isnt in constructor', function(){
    var SCP2 = require('../../../../app/helpers/executions/scp.coffee');
    try{
      new SCP2();
    }catch(error){
      error.should.be.equal('[SCP] servers parameter is required');
    }
  });

  it('should throw message error when servers were empty in constructor', function(){
    var SCP = require('../../../../app/helpers/executions/scp.coffee');
    try{
      new SCP([]);
    }catch(error){
      error.should.be.equal('[SCP] servers parameter is required');
    }
  });

  it('should instantiate SCP', function(){
    var SCP = require('../../../../app/helpers/executions/scp.coffee');
    var servers = ['127.0.0.1'];
    var scpPort = 2222;
    var scp = new SCP(servers, scpPort);
    should.exist(scp);
    servers.should.be.equal(scp.servers);
    scpPort.should.be.equal(scp.scpPort);
  });


  it('should throw exception when try connect SCP to server with no command', function(){
    this.timeout(10000);
    var SCP = require('../../../../app/helpers/executions/scp.coffee');
    var servers = ['127.0.0.1'];
    var scpPort = 2222;
    var scp = new SCP(servers, scpPort);
    should.exist(scp);
    servers.should.be.equal(scp.servers);
    scpPort.should.be.equal(scp.scpPort);

    try{
      scp.connect('root', '5tgbnhy6', null);
    }catch(error){
      error.should.be.equal('options parameter is required');
    }
  });

  it('should connect SCP to server', function(done){
    this.timeout(10000);
    var fs = require('fs');
    var SCP = require('../../../../app/helpers/executions/scp.coffee');
    var servers = '127.0.0.1';
    var scpPort = 2222;
    var scp = new SCP(servers, scpPort);

    should.exist(scp);
    servers.should.be.equal(scp.servers);
    scpPort.should.be.equal(scp.scpPort);

    var filename = 'test.txt'
    var request = {
      path:'/home/vagrant/'+filename,
      target: 'trash/'
    }

    if(fs.existsSync(request.target+filename)){
      var res = fs.unlinkSync(request.target+filename);
    }
    var exists = fs.existsSync(request.target+filename);
    //console.log("Exists? "+exists);
    exists.should.be.false;

    scp.connect('root', '5tgbnhy6', request);
    scp.on('end', function(err){
      var exists = fs.existsSync(request.target+filename);
      exists.should.be.true;
      done();
    })
  });
});